#ifndef _ROBOT_PARAM_H__
#define _ROBOT_PARAM_H__

#include <string>
#include <vector>

#include "xml_serdes.h"

using namespace std;

/** ロボットの姿勢パラメータ
 */
class RobotParam{
public:
	/** コンストラクタ
	 */
	RobotParam():
		time(0),
		coordinate("self"),
		shape_x0(0.0),
		shape_y0(0.0),
		shape_x1(0.0),
		shape_y1(0.0),
		mobility_x0  (0.0),
		mobility_y0  (0.0),
		mobility_the0(0.0),
		mobility_x1  (0.0),
		mobility_y1  (0.0),
		mobility_the1(0.0)
	{};

	/** デストラクタ
	 */
	~RobotParam(){};

	double time;         ///< 時刻(s)
	string coordinate;   ///< 座標系(self or 9)

	float shape_x0;      ///< 形 x座標0
	float shape_y0;      ///< 形 y座標0
	float shape_x1;      ///< 形 x座標1
	float shape_y1;      ///< 形 y座標1

	float mobility_x0;   ///< x座標0
	float mobility_y0;   ///< y座標0
	float mobility_the0; ///< 向き0
	float mobility_x1;   ///< x座標1
	float mobility_y1;   ///< y座標1
	float mobility_the1; ///< 向き1

	// type名
	static const char TYPE_ID[]; ///< タイプID文字列

	// 送受信メソッド
	/** データをXMLとして書き出す
	 *
	 * @param [out] os 書き出し先ストリーム
	 * @retval 成功時0, 失敗時は負の値を返す
	 */
	int writeToXml(std::ostream& os) const
	{
		return XmlSerDes::writeToXmlSub(os, *this, TYPE_ID);
	}

	/** XMLからデータを読み出し自分自身を更新する
	 *
	 * @param [in] is 読み出し元ストリーム
	 * @retval 成功時0, 失敗時は負の値を返す
	 */
	int readFromXml(std::istream& is)
	{
		return XmlSerDes::readFromXmlSub(is, *this, TYPE_ID);
	}

	/**
	 * \overload int writeToXml(std::string& s) const
	 * @param [out] s 書き出し先文字列
	 */
	int writeToXml(std::string& s) const
	{
		std::ostringstream os(s);
		return writeToXml(os);
	}

	/** XMLからデータを読み出し自分自身を更新する
	 *
	 * \overload int readFromXml(const std::string& s)
	 * @param [in] s 読み出し元文字列
	 */
	int readFromXml(const std::string& s)
	{
		std::istringstream is(s);
		return readFromXml(is);
	}

private:
	friend class boost::serialization::access;
	template <class Archive>
	void serialize(Archive& ar, const unsigned int /* version */){
		using namespace boost::serialization;

		ar & BOOST_SERIALIZATION_NVP(time);
		ar & BOOST_SERIALIZATION_NVP(coordinate);

		ar & BOOST_SERIALIZATION_NVP(shape_x0);
		ar & BOOST_SERIALIZATION_NVP(shape_x1);
		ar & BOOST_SERIALIZATION_NVP(shape_y0);
		ar & BOOST_SERIALIZATION_NVP(shape_y1);

		ar & BOOST_SERIALIZATION_NVP(mobility_x0);
		ar & BOOST_SERIALIZATION_NVP(mobility_x1);
		ar & BOOST_SERIALIZATION_NVP(mobility_y0);
		ar & BOOST_SERIALIZATION_NVP(mobility_y1);
		ar & BOOST_SERIALIZATION_NVP(mobility_the0);
		ar & BOOST_SERIALIZATION_NVP(mobility_the1);
	};
};

std::ostream& operator<<(std::ostream &os, RobotParam &param);

#endif //_ROBOT_PARAM_H__
