#ifndef  _SERDES_IF_H__
#define  _SERDES_IF_H__

#include <string>

#include "baseRTC.h"
#include "xml_serdes.h"


template <class TYPE>
class SerDesIF{
public:
	/** コンストラクタ
	 */
	SerDesIF();

	/** コンストラクタ
	 *
	 * @param[in] category コンポーネントカテゴリを指定 (参照渡し)
	 * @param[in] name コンポーネント名を指定 (参照渡し)
	 * @param[in] id コンポーネントIDを指定 (参照渡し)
	 */
	SerDesIF(const std::string &category, const std::string &name, const std::string &id);

	/** デストラクタ
	 */
	~SerDesIF();

	/** データをXML文字列へシリアライズする
	 *
	 * @param[out] xml 変換されたXML文字列を受け取るstringへのポインタを指定する
	 * @return 成功時は0, 失敗時はそれ以外を返す
	 */
	int SerializeToXML(std::string *xml);

	/** XML文字列からからデータを取りだす
	 *
	 * @param[in] xml 元データのXML文字列を指定する (コピーコスト削減のため参照渡し)
	 * @return 成功時は0, 失敗時はそれ以外を返す
	 */
	int DeserializeFromXML(const std::string &xml);

	/** ヘッダを設定する
	 *
	 * @param[in] category コンポーネントカテゴリを指定 (参照渡し)
	 * @param[in] name コンポーネント名を指定 (参照渡し)
	 * @param[in] id コンポーネントIDを指定 (参照渡し)
	 * @return 成功時は0, 失敗時はそれ以外を返す
	 */
	int SetHeader(const std::string &category, const std::string &name, const std::string &id, const std::string);

	/** ヘッダを取得する
	 *
	 * @param[in] category コンポーネントカテゴリを指定 (参照渡し)
	 * @param[in] name コンポーネント名を指定 (参照渡し)
	 * @param[in] id コンポーネントIDを指定 (参照渡し)
	 * @return 成功時は0, 失敗時はそれ以外を返す
	 */
	baseRTC::Header GetHeader(void);

	/** 送信用時刻を設定する
	 *
	 * @param[in] time 設定する時刻
	 * @return 設定した時刻を返す
	 */
	double SetTime(double time);


	TYPE            Data;   ///< データ

private:
	baseRTC::Header Header; ///< ヘッダ

};

template <class TYPE>
SerDesIF<TYPE>::
SerDesIF():
	Header("", TYPE::TYPE_ID, "", "", "")
{};

template <class TYPE>
SerDesIF<TYPE>::
SerDesIF(const std::string &category, const std::string &name, const std::string &id):
	Header(category, TYPE::TYPE_ID, name, id, "")
{};

template <class TYPE>
SerDesIF<TYPE>::
~SerDesIF()
{};

template <class TYPE>
int SerDesIF<TYPE>::
SerializeToXML(std::string *xml)
{
	using namespace XmlSerDes;
	std::ostringstream oss;

	Header.writeToXml(oss);
	Data.writeToXml(oss);

	*xml = oss.str();

	return 0;
};

template <class TYPE>
int SerDesIF<TYPE>::
DeserializeFromXML(const std::string &xml)
{
	using namespace XmlSerDes;
	std::istringstream iss(xml);

	Header.readFromXml(iss);
	Data.readFromXml(iss);

	return 0;
};

template <class TYPE>
int SerDesIF<TYPE>::
SetHeader(const std::string &category, const std::string &name, const std::string &id, const std::string tag)
{
	Header.category    = category;
	Header.sender_name = name;
	Header.id          = id;
	Header.tag         = tag;

	return 0;
}

template <class TYPE>
baseRTC::Header SerDesIF<TYPE>::
GetHeader(void)
{
	return Header;
}

template <class TYPE>
double SerDesIF<TYPE>::
SetTime(double time)
{
	Header.time = time;
	return Header.time; 
};

#endif //_SERDES_IF_H__
