#include <iostream>

#include "GridMap.h"

using namespace std;

const char GridMap::TYPE_ID[] = "GridMap";
#define map(x, y, nx, ny) map[(ny) * ((nx) - (x) - 1) + ((ny) - (y) - 1)]

int GridMap::
sizeX(void) const
{
	return (int)((area_x1 - area_x0 + size_x / 2.0f) / size_x);
}

int GridMap::
sizeY(void) const
{
	return (int)((area_y1 - area_y0 + size_y / 2.0f) / size_y);
}

int GridMap::
resize(void)
{
	int size;
	size = sizeX() * sizeY();
	map.resize(size);
	return size;
}

int GridMap::
Mapping(GridMap &src)
{
	if(map.empty()){ // 地図入力がない場合
		return -1;
	}
	if(src.map.empty()){ // 地図入力がない場合
		return -1;
	}

	float x, y;
	int nx, ny, xi, yi;
	int in_x, in_y, out_x, out_y;

	out_x = sizeX();
	out_y = sizeY();
	if((int)map.size() < (out_x * out_y)){
		cerr << "map size too small." << endl;
		return -1;
	}

	in_x = src.sizeX();
	in_y = src.sizeY();

	for(y = area_y0 + size_y / 2.0f, yi = 0; yi < out_y; y += size_y, yi ++){
		for(x = area_x0 + size_x / 2.0f, xi = 0; xi < out_x; x += size_x, xi ++){
			nx = (x - src.area_x0) / src.size_x;
			ny = (y - src.area_y0) / src.size_y;
			if ((nx >= 0)&&(nx < in_x)&&(ny >= 0)&&(ny < in_y)){
				map[out_x * yi + xi] = src.map[in_x * ny + nx];
			} else {
				map[out_x * yi + xi] = 0.5f;
			}
		}
	}

	return 0;
}

int GridMap::
FillRectangle(float x0, float y0, float x1, float y1, float val)
{
	if(map.empty()){ // 地図入力がない場合
		return -1;
	}

	float x, y;
	int nx, ny, out_x, out_y;

	out_x = sizeX();
	out_y = sizeY();
	if((int)map.size() < (out_x * out_y)){
		cerr << "map size too small." << endl;
		return -1;
	}

	for(y = y0; y < y1; y += size_y){
		for(x = x0; x < x1; x += size_x){
			nx = (x - area_x0) / size_x;
			ny = (y - area_y0) / size_y;
			if ((nx >= 0)&&(nx < out_x)&&(ny >= 0)&&(ny < out_y)){
				map(nx,ny,out_x,out_y) = val;
			}
		}
	}

	return 0;
}

int GridMap::
DrawPoint(float x, float y, float val)
{
	if(map.empty()){ // 地図入力がない場合
		return -1;
	}

	int nx, ny, out_x, out_y;

	out_x = sizeX();
	out_y = sizeY();
	if((int)map.size() < (out_x * out_y)){
		cerr << "map size too small." << endl;
		return -1;
	}	
	nx = (x - area_x0) / size_x;
	ny = (y - area_y0) / size_y;
	if ((nx >= 0)&&(nx < out_x)&&(ny >= 0)&&(ny < out_y)){
		map(nx,ny,out_x,out_y) = val;
	}

	return 0;
}

int GridMap::
DrawLine(float x0, float y0, float x1, float y1, float val)
{
	if(map.empty()){ // 地図入力がない場合
		return -1;
	}

	float x, y, dx, dy, slope;
	int nx, ny, out_x, out_y;

	out_x = sizeX();
	out_y = sizeY();
	if((int)map.size() < (out_x * out_y)){
		cerr << "map size too small." << endl;
		return -1;
	}

	dx = x1 - x0;
	dy = y1 - y0;
	if ((dx == 0.0f)&&(dy == 0.0f)) return -1; // 共に0ならば戻る
	if (fabs(dx) > fabs(dy)){ // xを増減する
		slope = dy / dx;
		if (x0 > x1){
			x = x1; x1 = x0; x0 = x;
			y = y1;
		} else {
			x = x0;
			y = y0;
		}
		for(; x < x1; x += size_x){
			nx = (x - area_x0) / size_x;
			ny = (y - area_y0) / size_y;
			if ((nx >= 0)&&(nx < out_x)&&(ny >= 0)&&(ny < out_y)){
				map(nx,ny,out_x,out_y) = val;
			}
			y += slope * size_x;
		}
	} else {
		slope = dx / dy;
		if (y0 > y1){
			y = y1; y1  = y0; y0 = y;
			x = x1;
		} else {
			y = y0;
			x = x0;
		}
		for(; y < y1; y += size_y){
			nx = (x - area_x0) / size_x;
			ny = (y - area_y0) / size_y;
			if ((nx >= 0)&&(nx < out_x)&&(ny >= 0)&&(ny < out_y)){
				map(nx,ny,out_x,out_y) = val;
			}
			x += slope * size_y;
		}
	}

	return 0;
}

ostream&
operator<<(ostream &os, GridMap &map)
{
	os << fixed << endl;
	os << "time:       " << map.time << endl;
	os << "id:         " << map.id << endl;
	os << "coordinate: " << map.coordinate << endl;
	os << "area:       " << "((" << map.area_x0 << ", " << map.area_y0 << ")-(" << map.area_x1 << ", " << map.area_y1 << "))" << endl;
	os << "size:       " << "(" << map.size_x << ", " << map.size_y << ")" << endl;
	os << "range:      " << "(" << map.range_min << " - " << map.range_max << "," << map.range_step << ")" << endl;
	os << "map.size:   " << map.map.size() << endl;

	return os;
}
