#ifndef  _ENV_RECOGNITION_CONFIG_H__
#define  _ENV_RECOGNITION_CONFIG_H__

#include <map>
#include <string>

#include "Fusion.h"
#include "GridMap.h"

using namespace std;


// コンフィグファイルからの読み込みなので, XMLパーサ使用
class EnvRecognitionConfig{
public:
	/** コンストラクタ
	 */
	EnvRecognitionConfig();

	/** デストラクタ
	 */
	~EnvRecognitionConfig();

	/** データをファイルへ書き出す
	 *
	 * @param[in] filename 書き込むファイル名
	 * @return 成功時は0, 失敗時はそれ以外を返す
	 */
	int SaveToFile(string filename);

	/** ファイルらからデータを読み込む
	 *
	 * @param[in] xml 読み込むファイル名
	 * @return 成功時は0, 失敗時はそれ以外を返す
	 */
	int LoadFromFile(string filename);

	/** ostream へ << 演算子で内容出力用
	 *
	 * cout << Config; と言った使い方を可能にする.
	 */
	friend std::ostream& operator<<(std::ostream& os, const EnvRecognitionConfig& urg);

	string category; ///< カテゴリ名
	string name;     ///< コンポーネント名
	string id;       ///< コンポーネントID

	vector<string> receive; ///< 受信するデータのリスト (送信元ID)

	int input_number; ///< 入力ポート数

	double timeout; ///< タイムアウト設定

	GridMap grid_map; ///< 存在確率マップ (area, grid_size, rangeのみ使用)
	GridMap work_map; ///< 過去のデータと統合するための確率マップ (area, grid_size, rangeのみ使用)

	float memory_period;   ///< 統合用パラメータ データの保持時間
	float memory_decrease; ///< 統合用パラメータ 時間経過によるデータの減衰率

	bool gui; ///< GUI表示設定

private:
};



#endif //_ENV_RECOGNITION_CONFIG_H__
