// Fusion.cpp : 実装ファイル
#include <iostream>
#include "Fusion.h"
#include <sys/time.h>
#include <math.h>

#define map(x,y,max_x,max_y)	map[(max_y)*((max_x)-(x)-1)+((max_y)-(y)-1)]

// Fusion
CFusion::CFusion()
{
}

CFusion::~CFusion()
{
}

// Fusion メッセージ ハンドラ

int CFusion::init()
{
	setMemoryPara(1.0,1.0);
	return 0;
}

int CFusion::setMemoryPara(float period, float decrease)
{
	CFusion::period = period;
	CFusion::decrease = decrease;
	return 0;
}


int CFusion::
setGridMap(GridMap *sensor_data)
{
	int i;
	struct timeval tv;
	gettimeofday(&tv,NULL);
	double now = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0);

	//printf("Fusion add data ID0: %s\n", sensor_data->id.c_str());

	stored_data.push_back(*sensor_data);

	i = 0;
	while(i < (int)(stored_data.size())){
		if (stored_data[i].time < (now - period)){
			stored_data.erase(stored_data.begin()+i);
		} else {
			i ++;
		}
	}
	return 0;
}

int CFusion::
setWorkMap(GridMap *grid_map)
{
	work_map = grid_map;

	return 0;
}


int CFusion::
setSelfLocalization(Localization *self_loc)
{
	int i = 0;
	struct timeval tv;
	gettimeofday(&tv,NULL);
	double now = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0);

	stored_self_loc.push_back(*self_loc);
	while(i < (int)(stored_self_loc.size())){
		if (stored_self_loc[i].time < (now - period)){
			stored_self_loc.erase(stored_self_loc.begin()+i);
		} else {
			i ++;
		}
	}
	printf("stored POSITION data size:%d\n", stored_self_loc.size());
	return 0;
}

int CFusion::
getEstimatedLocalization(Localization *self_loc, double time)
{
	int i;
	Localization *sl0, *sl1;
	float r0, r1;

	for(i = 0;i < (int)(stored_self_loc.size());i ++){
		if ((stored_self_loc[i].time > time)||(i == (int)(stored_self_loc.size())-1)){
			if (i == (int)(stored_self_loc.size())-1) {
				sl0 = sl1 = &stored_self_loc[i];
				r0 = 1.0f;
				r1 = 1.0f - r0;
			} else {
				sl0 = &stored_self_loc[i-1];
				sl1 = &stored_self_loc[i  ];
				r0 = (sl1->time - time) / (sl1->time - sl0->time);
				r1 = 1.0f - r0;
			}
			self_loc->time        = time;
			self_loc->pos_x       = r0 * sl0->pos_x       + r1 * sl1->pos_x      ;
			self_loc->pos_y       = r0 * sl0->pos_y       + r1 * sl1->pos_y      ;
			self_loc->pos_the     = r0 * sl0->pos_the     + r1 * sl1->pos_the    ;
			self_loc->pos_err_x   = r0 * sl0->pos_err_x   + r1 * sl1->pos_err_x  ;
			self_loc->pos_err_y   = r0 * sl0->pos_err_y   + r1 * sl1->pos_err_y  ;
			self_loc->pos_err_the = r0 * sl0->pos_err_the + r1 * sl1->pos_err_the;
			self_loc->vel_x       = r0 * sl0->vel_x       + r1 * sl1->vel_x      ;
			self_loc->vel_y       = r0 * sl0->vel_y       + r1 * sl1->vel_y      ;
			self_loc->vel_the     = r0 * sl0->vel_the     + r1 * sl1->vel_the    ;
			return 0;
		}
	}
	return -1;
}

/*
 * 画像データの変換
 */
int CFusion::
transform(GridMap *src, float dx, float dy, float dthe, GridMap *dst)
{
	float x, y, sx, sy, dx0, dy0, scx, scy, dcx, dcy, s, c;
	int xi, yi, nx, ny, dst_x, dst_y, src_x, src_y;

	src_x = src->sizeX();	// 入力データのピクセル数
	src_y = src->sizeY();
	if ((int)src->map.size() < (src_x * src_y)){ // 十分な領域があるか
		cerr << "Error : not enough source map memory in transform function" << endl;
	}
	dst_x = dst->sizeX();	// 出力データのピクセル数
	dst_y = dst->sizeY();
	if ((int)dst->map.size() < (dst_x * dst_y)){ // 十分な領域があるか
		cerr << "Error : not enough distination map memory in transform function" << endl;
	}

	scx = (src->area_x0 + src->area_x1) / 2.0f;	// 変換元の座標中心
	scy = (src->area_y0 + src->area_y1) / 2.0f;
	dcx = (dst->area_x0 + dst->area_x1) / 2.0f;	// 変換先の座標中心
	dcy = (dst->area_y0 + dst->area_y1) / 2.0f;

	c = cos(dthe);	// 予めsin,cosを計算しておく
	s = sin(dthe);

	// 変換先のピクセルごとに変換
	for(y = dst->area_y0 + dst->size_y / 2.0f, yi = 0; yi < dst_y; y += dst->size_y, yi ++){
		for(x = dst->area_x0 + dst->size_x / 2.0f, xi = 0; xi < dst_x; x += dst->size_x, xi ++){
			dx0 = x - dcx;	// 変換先の回転中心kからの距離
			dy0 = y - dcy;
			sx = dx0 * c - dy0 * s + scx;
			sy = dx0 * s + dy0 * c + scy;
			nx = (sx - src->area_x0) / src->size_x;
			ny = (sy - src->area_y0) / src->size_y;
			if ((nx >= 0)&&(nx < src_x)&&(ny >= 0)&&(ny < src_y)){
				dst->map(xi,yi,dst_x,dst_y) = src->map(nx,ny,src_x,src_y);
			} else {
				dst->map(xi,yi,dst_x,dst_y) = 0.5f;
			}
		}
	}
	return 0;
}

// データを統合したグリッドマップを作成
int CFusion::
getMergedGridMap(GridMap *result)
{
	int i, j;
	double time, now;
	Localization self_loc, now_loc;
	GridMap gmap;
	static float x0 = 0.0f, y0 = 0.0f;
	float dx, dy, dthe, range_x, range_y, ratio;
	int is_change_base = 0;
	struct timeval tv;
	
	gettimeofday(&tv,NULL);
	now = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0);

	gmap.area_x0    = work_map->area_x0   ; gmap.area_y0    = work_map->area_y0  ;
	gmap.area_x1    = work_map->area_x1   ; gmap.area_y1    = work_map->area_y1  ;
	gmap.size_x     = work_map->size_x    ; gmap.size_y     = work_map->size_y   ;
	gmap.range_min  = work_map->range_min ; gmap.range_max  = work_map->range_max;
	gmap.range_step = work_map->range_step;
	gmap.map.clear();
	gmap.resize();

	if (stored_self_loc.size() >= 1){
		now_loc = stored_self_loc[stored_self_loc.size() - 1];
	}

	range_x = work_map->area_x1 - work_map->area_x0;
	range_y = work_map->area_y1 - work_map->area_y0;

	if (fabs(now_loc.pos_x - x0) > range_x / 2.0f){	// 作業領域のヘリまで移動した場合
		x0 = now_loc.pos_x;
		is_change_base = 1;
	} else if (fabs(now_loc.pos_x - x0) > range_x / 4.0f){	// 作業領域のヘリまで移動した場合
		if (now_loc.pos_x > x0)	x0 += range_x / 4.0f;
		else					x0 -= range_x / 4.0f;
		is_change_base = 1;
	}
	if (fabs(now_loc.pos_y - y0) > range_y / 2.0f){	// 作業領域のヘリまで移動した場合
		y0 = now_loc.pos_y;
		is_change_base = 1;
	} else if (fabs(now_loc.pos_y - y0) > range_y / 4.0f){	// 作業領域のヘリまで移動した場合
		if (now_loc.pos_y > y0)	y0 += range_y / 4.0f;
		else					y0 -= range_y / 4.0f;
		is_change_base = 1;
	}

	// 作業領域を変更する場合
	// 一旦gmapに移動したのち、workspaceに戻す
	if (is_change_base){	// 作業領域のヘリまで移動した場合
		dx = x0 - (gmap.area_x1 + gmap.area_x0) / 2.0f;
		dy = y0 - (gmap.area_y1 + gmap.area_y0) / 2.0f;
		gmap.area_x0 += dx; gmap.area_y0 += dy;
		gmap.area_x1 += dx; gmap.area_y1 += dy;
		transform(work_map, 0.0f, 0.0f, 0.0f, &gmap);

		work_map->area_x0 += dx; work_map->area_y0 += dy;
		work_map->area_x1 += dx; work_map->area_y1 += dy;
		for(i = 0; i < (int)work_map->map.size(); i++){
			work_map->map[i] = gmap.map[i];
		}
		is_change_base = 0;
	}

	// 徐々にデータの信頼性を下げる
	ratio = max((float)(1.0f - (now - work_map->time) * decrease), 0.0f);
	printf(">>> ratio: %f\n",ratio);
	for(j = 0; j < (int)work_map->map.size(); j++){
		work_map->map[j] = (work_map->map[j] - 0.5f) * ratio + 0.5f;
	}

	// センサに保存されたデータを重ね合わせる
	for(i = 0;i < (int)stored_data.size();i ++){
		time = stored_data[i].time;
		ratio = 1.0f - (now - time) * decrease;
		if (ratio <= 0.0f){
			continue;
		}
		printf("ratio[%d]: %f\n",i,ratio);

		// 保存されているデータを現在の位置にマッピングする
		if (getEstimatedLocalization(&self_loc, time) != -1){
			dx   = self_loc.pos_x  ;
			dy   = self_loc.pos_y  ;
			dthe = self_loc.pos_the;
			// センサデータを絶対座標に変換する
			transform(&stored_data[i], dx, dy, dthe, &gmap);
		} else {
			transform(&stored_data[i], 0, 0, 0, &gmap);
		}
		// 0.5(unknown)から最も遠い値を選定する．
		for(j = 0; j < (int)work_map->map.size(); j++){
			if (fabs(work_map->map[j] - 0.5f) < (fabs(gmap.map[j] - 0.5f) * ratio)){
				work_map->map[j] = (gmap.map[j] - 0.5f) * ratio + 0.5f;
			}
		}
	}

	work_map->time = now;
//	transform(work_map, 0, 0, 0, result);
	transform(work_map, -now_loc.pos_x, -now_loc.pos_y, -now_loc.pos_the, result);
//	for(j = 0; j < (int)work_map->map.size(); j++){
//		//		result->map[j] = stored_data[0].map[j];
//		result->map[j] = gmap.map[j];
//	}

	result->time = now;
	printf("stored_data: %d\n",stored_data.size());
	stored_data.clear();

	//temp for check
//	for(j = 0; j < (int)result->map.size(); j++){
//		result->map[j] = 0.1f;
//	}
//	printf("x0: %f, y0:%f pos_the:%f\n",x0 ,y0, now_loc.pos_the);

	return 0;
}

int CFusion::close()
{
	return 0;
}

