// -*- C++ -*-
/*!
 * @file  LocalPathPlan.h * @brief LocalPathPlan * @date  $Date$ 
 *
 * $Id$ 
 */
#ifndef LOCAL_PATH_PLAN_H
#define LOCAL_PATH_PLAN_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

#include "LocalPathPlanConfig.h"
#include "SerDesIF.h"
#include "GridMap.h"
#include "Path.h"
#include "navigation_data.h"

#include "Potential.h"

#define NAME_LENGTH 128

class LocalPathPlan  : public RTC::DataFlowComponentBase
{
 public:
  LocalPathPlan(RTC::Manager* manager);
  ~LocalPathPlan();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
  virtual RTC::ReturnCode_t onInitialize();

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

	static const char Category[];
	static const char Name[];

 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
	std::vector<RTC::TimedString *> m_in;
	std::vector<RTC::InPort<RTC::TimedString> *> m_inIn;

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
	RTC::TimedString m_out;
	RTC::OutPort<RTC::TimedString, RTC::NullBuffer> m_outOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">

  // </rtc-template>

private:

	/** リスク管理モジュールからの受信データ (XML) を処理する
	 *
	 * リスク管理モジュールから受信したリスクマップを処理する
	 *
	 * @param[in] recv 受信したXML文字列 (コピーコスト削減のため参照渡し)
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t ProcessRiskMapData(const std::string &recv);

	/** 統合結果を送信する
	 *
	 * 統合されたセンサデータをXMLに変換して, 次のコンポーネントへ送信する
	 *
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t SendResult(void);

	/** 大経路計画からの経路情報 (XML) を処理する
	 *
	 * 大経路計画から受信した経路情報を処理する
	 *
	 * @param[in] recv 受信したXML文字列 (コピーコスト削減のため参照渡し)
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t ProcessPathNodesData(const std::string &recv);

	/** 現在位置を (XML) を処理する
	 *
	 * 現在位置を処理する
	 *
	 * @param[in] recv 受信したXML文字列 (コピーコスト削減のため参照渡し)
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t ProcessSelfLocationData(const std::string &recv);

	std::string filename;
	LocalPathPlanConfig *Config;   ///< 設定

	GridMap					View;		///< 表示用データ受け取り用クラス
	SerDesIF<Path>			*PathData;	///< 大経路計画受信用クラス
	SerDesIF<GridMap>		*RiskMap;	///< リスクグリッドマップ受信用クラス
	SerDesIF<Localization>	*PosData;	///< オドメトリ受信用クラス

	SerDesIF<GridMap>   *PotentialMap;  ///< 結果送信データ用クラス
	SerDesIF<Path>      *ResultPath;    ///< 結果送信データ用クラス

//	SerDesIF<GridMap>            *RiskMap;       ///< リスクグリッドマップ受信用クラス
//	SerDesIF<NavData::PathNodes> *PathNodesData; ///< 大経路計画受信用クラス

	CPotential Potential;
};


extern "C"
{
  void LocalPathPlanInit(RTC::Manager* manager);
};

#endif // LOCAL_PATH_PLAN_H
