#include <iostream>
#include <string>
#include <sstream>

#include <sys/time.h>

#include "LocalPathPlanConfig.h"

// XML パーサ用
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include "XmlHandler.h"

using namespace std;

LocalPathPlanConfig::
LocalPathPlanConfig():
	category(""),
	name    (""),
	id      ("")
{
	input_number = 0;
	gui = false;
}

LocalPathPlanConfig::
~LocalPathPlanConfig()
{
}


// TODO: 送信側 (コンフィグ保存) の実装は必要？
int LocalPathPlanConfig::
SaveToFile(string filename)
{
	return 0;
}

int LocalPathPlanConfig::
LoadFromFile(string filename)
{

	vector<string> str;
	
	InitXmlParser();

	if (!XmlParser(filename, "rtc_config", "name", &str)){
		cerr << "Cannot find [name] in configuration file" << endl;
		return -1;
	}
	name = str[0];

	if (!XmlParser(filename, "rtc_config", "category", &str)){
		cerr << "Cannot find [category] in configuration file" << endl;
		return -1;
	}
	category = str[0];

	if (!XmlParser(filename, "id", "", &str)){
		cerr << "Cannot find <id> in configuration file" << endl;
		return -1;
	}
	id = str[0];

	if (!XmlParser(filename, "receive", "id", &str)){
		cerr << "Cannot find <receive> in configuration file" << endl;
		return -1;
	}
	receive = str;

	if (!XmlParser(filename, "input_number", "", &str)){
		cerr << "Cannot find <input_number> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%d", &input_number);

	if (!XmlParser(filename, "timeout", "", &str)){
		cerr << "Cannot find <timeout> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%lf", &timeout);

	if (!XmlParser(filename, "area", "", &str)){
		cerr << "Cannot find <area> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f,%f,%f,%f", &grid_map.area_x0, &grid_map.area_y0, &grid_map.area_x1, &grid_map.area_y1);

	if (!XmlParser(filename, "grid_size", "", &str)){
		cerr << "Cannot find <grid_size> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f,%f", &grid_map.size_x, &grid_map.size_y);

	if (!XmlParser(filename, "range", "", &str)){
		cerr << "Cannot find <range> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f,%f,%f", &grid_map.range_min, &grid_map.range_max, &grid_map.range_step);

	if (!XmlParser(filename, "slope", "", &str)){
		cerr << "Cannot find <slope> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f", &slope);

	if (!XmlParser(filename, "goal_radius", "", &str)){
		cerr << "Cannot find <goal_radius> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f", &goal_radius);

	if (!XmlParser(filename, "gui", "", &str)){
		cerr << "Cannot find <gui> in configuration file" << endl;
		return -1;
	}
	if (str[0] == "ON"){
		gui = true;
	} else{
		gui = false;
	}

	TerminateXmlParser();

	return 0;
}

std::ostream&
operator<<(std::ostream& os, const LocalPathPlanConfig& conf)
{
	int i;

	os << "[LocalPathPlanConfig]" << fixed << endl;
	os << "category:       " << conf.category << endl;
	os << "name:           " << conf.name << endl;
	os << "id:             " << conf.id << endl;
	os << "input_number:   " << conf.input_number << endl;
	os << "timeout:        " << conf.timeout << endl;
	os << "slope:          " << conf.slope << endl;
	os << "gui:            " << conf.gui << endl;

	os << "receive: ";
	for(i=0; i<(int)conf.receive.size(); i++){
		if(i != 0){
			os << ", ";
		}
		os << conf.receive[i];
	}
	os << endl;

	return os;
}

