#include "LocalPathPlanDialog.h"
#include "LocalPathPlanDialog_int.h"
#include <QtGui>

#include "GridMap.h"
#include "Path.h"

pthread_mutex_t dialog_mutex;
QApplication *app = NULL;
int data_x = 0, data_y = 0;
float *draw_data = NULL;

GridMap gmap;

static Path    draw_path;     ///< 重ね合わせ用に保持

LocalPathPlanWidget::
LocalPathPlanWidget(QWidget *parent): QWidget(parent)
{
	setWindowTitle(tr("LocalPathPlan Dialog"));
	setPalette(QPalette(QColor(255, 255, 255)));
	setAutoFillBackground(true);
}

void LocalPathPlanWidget::
set_value(GridMap *grid_map)
{
	pthread_mutex_lock(&dialog_mutex);

	gmap = *grid_map;

	int x, y;
	x = (int)((grid_map->area_x1 - grid_map->area_x0 + grid_map->size_x / 2.0f) / grid_map->size_x);
	y = (int)((grid_map->area_y1 - grid_map->area_y0 + grid_map->size_y / 2.0f) / grid_map->size_y);

	if ((data_x != x)||(data_y != y)){
		if (draw_data != NULL){
			free(draw_data);
		}
		data_x = x;
		data_y = y;
		draw_data = (float *)malloc(x * y * sizeof(float));
	}
	for(int i = 0; i < (x * y); i ++){
		draw_data[i] = grid_map->map[i];
	}
	
	pthread_mutex_unlock(&dialog_mutex);
	update();
}

void LocalPathPlanWidget::
paintEvent(QPaintEvent *event)
{
	QPainter painter(this);
	painter.setPen(Qt::NoPen);

	int x0, y0, x1 = 0, y1 = 0, red, green, blue;
	float rx, ry;
	int no = 0;

	int i;

	pthread_mutex_lock(&dialog_mutex);

	// グリッドマップ描画
	rx = (float)rect().width()  / data_x;
	ry = (float)rect().height() / data_y;

	for(int y = 0; y < data_y; y ++){
		for(int x = 0; x < data_x; x ++){
			x0 = (int)((x  ) * rx), y0 = (int)((y  ) * ry);
			x1 = (int)((x+1) * rx), y1 = (int)((y+1) * ry);
			red   = 0xff * draw_data[no];
			green = 0xff * draw_data[no];
			blue  = 0xff * draw_data[no];
			no ++;
			painter.setBrush(QColor(red, green, blue, 255));
			painter.drawRect(x0, y0, (int)rx+1, (int)ry+1);
		}
	}

	// 経路描画
	// TODO (draw_path を painterに描画する)
	rx = static_cast<double>(rect().width() ) / static_cast<double>(gmap.area_x1 - gmap.area_x0);
	ry = static_cast<double>(rect().height()) / static_cast<double>(gmap.area_y1 - gmap.area_y0);

	if(static_cast<int>(draw_path.path.size()) > 1){
		for(i=1; i<static_cast<int>(draw_path.path.size()); i++){
			Node s, e;
			s = draw_path.path[i-1];
			e = draw_path.path[i];

			x0 = (gmap.area_x0 - s.node.x) * rx * -1;
			x1 = (gmap.area_x0 - e.node.x) * rx * -1;
                           
			y0 = (gmap.area_y1 - s.node.y) * ry;
			y1 = (gmap.area_y1 - e.node.y) * ry;

			painter.setPen(Qt::red);
			painter.drawLine(x0, y0, x1, y1);
		}
	}

	pthread_mutex_unlock(&dialog_mutex);
}

LocalPathPlanWidget *envRecognitionWidget = NULL;

void*
windowThread(void* param)
{
	int argc = 1;
	char *argv[] = {};

	pthread_mutex_init( &dialog_mutex, NULL );
	app = new QApplication(argc, argv);
	envRecognitionWidget = new LocalPathPlanWidget();
	envRecognitionWidget->show();
	app->exec();

	delete envRecognitionWidget;
	delete app;

	return 0;
}

int
initialize_window(void)
{
//#define PATH_DRAW_TEST
#undef PATH_DRAW_TEST
#if defined(PATH_DRAW_TEST)
	std::cerr << "test path for draw_path set" << std::endl;
	draw_path.path.push_back(Node(0,0,0));
	draw_path.path.push_back(Node(1,1,0));
	draw_path.path.push_back(Node(1,1.5,0));
	draw_path.path.push_back(Node(1.5,1.5,0));
	draw_path.path.push_back(Node(1.5,-1.5,0));
	draw_path.path.push_back(Node(-1.5,-1.5,0));
	draw_path.path.push_back(Node(-1.5,1.5,0));
	draw_path.path.push_back(Node(-0.5,0.5,0));
#endif

	return 0;
}

void
close_window(void)
{
	if(draw_data != NULL){
		free(draw_data);
		draw_data = NULL;
	}

	app->quit();

	return;
}

int
set_value(GridMap *grid_map)
{
	if (envRecognitionWidget != NULL){
		envRecognitionWidget->set_value(grid_map);
	}

	return 0;
}

int
set_value(Path *path)
{
#if defined(PATH_DRAW_TEST)
	std::cerr << "set_value(Path*): " << path->path.size() << std::endl;
#else
	draw_path = *path;
//	std::cerr << draw_path << std::endl;
#endif

	return 0;
}

